@@ -2,14 +2,11 @@
 Likelihood evaluation
 """
 
-from typing import Callable, List, Optional, Union
+from typing import List, Optional, Union
 
 import numpy as np
-import torch
 from scipy.linalg import eigh, eigh_tridiagonal
-from scipy.linalg.lapack import dpotrf
-from torch.distributions import MultivariateNormal
-from torch.distributions.normal import Normal
+from scipy.linalg.lapack import dpotrf, dpotrs
 
 from tripy.utils import (
     _cast_scalar_to_array,
@@ -473,6 +470,46 @@ def _kron_loglike_ND_tridiag(
     return -0.5 * np.prod(Nd) * np.log(2 * np.pi) - 0.5 * logdet_C - 0.5 * ySy
 
 
+def _loglike_normal(y, std):
+    """
+
+    Args:
+        y:
+        std:
+
+    Returns:
+
+    """
+
+    return -0.5 * (y / std) ** 2 - np.log(std * np.sqrt(2 * np.pi))
+
+
+def _loglike_multivariate_normal(y, cov):
+    """
+    Calculate the multivariate normal loglikelihood
+
+    Args:
+        y:
+        cov:
+
+    Returns:
+
+    """
+    N = y.size
+
+    # Lower Cholesky factor
+    L = np.linalg.cholesky(cov)
+
+    # Solve factorized system
+    x, info = dpotrs(L, y, lower=True, overwrite_b=False)
+    ySy = np.sum(y * x)
+
+    # Logdeterminant
+    logdet = np.sum(2 * np.log(np.diag(L)))
+
+    return -0.5 * (logdet + ySy + N * np.log(2 * np.pi))
+
+
 def log_likelihood_linear_normal(
     y: np.ndarray,
     K: Optional[Union[int, float, np.ndarray]],
@@ -516,7 +553,6 @@ def log_likelihood_linear_normal(
     # Pre-process
     y = y.ravel()
     N = y.size
-    y = torch.tensor(y)
 
     # Check that model uncertainty covariance matrix is supplied if `y_model`
     # is not None. Scale model uncertainty covariance by model prediction.
@@ -534,7 +570,7 @@ def log_likelihood_linear_normal(
     if K is None:
         if std_meas is not None:
             std_meas = _cast_scalar_to_array(std_meas, N).ravel()
-            dist = Normal(torch.zeros(N), torch.tensor(std_meas))
+            return _loglike_normal(y, std_meas)
         else:
             raise ValueError("At least one of `K`, `std_meas` expected to be not None")
 
@@ -542,89 +578,6 @@ def log_likelihood_linear_normal(
     else:
         if std_meas is not None:
             std_meas = _cast_scalar_to_array(std_meas, N).ravel()
-            dist = MultivariateNormal(
-                torch.zeros(N), torch.tensor(K + np.diag(std_meas ** 2))
-            )
+            return _loglike_multivariate_normal(y, K + np.diag(std_meas ** 2))
         else:
-            dist = MultivariateNormal(
-                torch.zeros(N), torch.tensor(K) + torch.ones(N) * jitter
-            )
-
-    return dist.log_prob(y).sum().detach().cpu().numpy()
-
-
-def log_likelihood_reference(
-    theta: np.ndarray,
-    x_meas: np.ndarray,
-    physical_model: Callable[[np.ndarray], np.ndarray],
-    k_cov_mx: Optional[np.ndarray] = None,
-    e_cov_mx: Optional[np.ndarray] = None,
-) -> np.ndarray:
-    """
-    Reference log-likelihood function for the following model,
-    for testing:
-
-    ``X_model = K*physical_model_fun(theta) + E``
-
-    where
-        * ``K`` ~ MVN(mean=1, covariance=K)
-        * ``E`` ~ MVN(mean=0, covariance=e_cov_mx)
-        * ``MVN()`` multivariate normal distribution
-
-    ``physical_model_fun()`` has no additional uncertainty, e.g. surrogate model
-    uncertainty.
-
-    Notes:
-        * This function is taken from `taralli` (https://gitlab.com/tno-bim/taralli/)
-
-    Args:
-        theta: value of the parameter(s) to be estimated in the Bayesian inference,
-            `shape [1, K].
-        x_meas: measurements in a [T, S] shape. T corresponds to the time space and S
-            to the physical and quantity space.
-        physical_model: function that takes as input theta and gives out y_model.
-        k_cov_mx: covariance matrix of `K`, shape [T*S, T*S].
-        e_cov_mx: covariance matrix of `elastic_mod`, shape [T*S, T*S].
-
-    Returns:
-        log_like: value of the log-likelihood evaluated at theta.
-
-    """
-    # ..................................................................................
-    # Pre-process
-    # ..................................................................................
-    x_meas = x_meas.ravel()
-    theta = np.atleast_2d(theta)
-    n_theta = theta.shape[0]
-
-    x_meas_n_elements = x_meas.size
-    k_cov_mx_zero_flag = False
-
-    if k_cov_mx is None:
-        k_cov_mx = np.zeros((x_meas_n_elements, x_meas_n_elements))
-        kph_cov_mx = k_cov_mx
-        k_cov_mx_zero_flag = True
-
-    if e_cov_mx is None:
-        e_cov_mx = np.zeros((x_meas_n_elements, x_meas_n_elements))
-
-    # ..................................................................................
-    # Calculate likelihood value(s)
-    # ..................................................................................
-    log_like = np.empty(n_theta)
-    for ii, theta_row in enumerate(theta):
-        x_ph = physical_model(theta_row).ravel()
-        x_ph_vector = x_ph.reshape(-1)
-        x_ph_diag_mx = np.diag(x_ph_vector)
-
-        x_diff = torch.tensor(x_meas - x_ph)
-
-        # covariance matrix of K*physical_model(theta)
-        if not k_cov_mx_zero_flag:
-            kph_cov_mx = np.matmul(x_ph_diag_mx, np.matmul(k_cov_mx, x_ph_diag_mx))
-
-        mvn = MultivariateNormal(
-            torch.zeros(len(x_ph_vector)), torch.tensor(kph_cov_mx + e_cov_mx)
-        )
-        log_like[ii] = mvn.log_prob(x_diff).detach().cpu().numpy()
-    return log_like
+            return _loglike_multivariate_normal(y, K + np.ones(N) * jitter)
